//============================================================================
//
// Copyright (C) 2002-2005  David Schneider, Lars Kdderitzsch
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//============================================================================

package com.atlassw.tools.eclipse.checkstyle.builder;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.filebuffers.FileBuffers;
import org.eclipse.core.filebuffers.ITextFileBufferManager;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.internal.corext.util.CodeFormatterUtil;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.osgi.util.NLS;
import org.eclipse.ui.texteditor.MarkerUtilities;

import com.atlassw.tools.eclipse.checkstyle.CheckstylePlugin;
import com.atlassw.tools.eclipse.checkstyle.Messages;
import com.atlassw.tools.eclipse.checkstyle.config.ICheckConfiguration;
import com.atlassw.tools.eclipse.checkstyle.config.meta.MetadataFactory;
import com.atlassw.tools.eclipse.checkstyle.config.meta.RuleMetadata;
import com.atlassw.tools.eclipse.checkstyle.util.CheckstyleLog;
import com.atlassw.tools.eclipse.checkstyle.util.CheckstylePluginException;
import com.atlassw.tools.eclipse.checkstyle.util.CustomLibrariesClassLoader;
import com.puppycrawl.tools.checkstyle.Checker;
import com.puppycrawl.tools.checkstyle.api.AuditEvent;
import com.puppycrawl.tools.checkstyle.api.AuditListener;
import com.puppycrawl.tools.checkstyle.api.CheckstyleException;
import com.puppycrawl.tools.checkstyle.api.Filter;
import com.puppycrawl.tools.checkstyle.api.SeverityLevel;

/**
 * Performs checking on Java source code.
 */
public class Auditor
{
    // =================================================
    // Public static final variables.
    // =================================================

    // =================================================
    // Static class variables.
    // =================================================

    /** The interval for updating the task info. */
    private static final int MONITOR_INTERVAL = 10;

    // =================================================
    // Instance member variables.
    // =================================================

    /** The check configuration the auditor uses. */
    private ICheckConfiguration mCheckConfiguration;

    /** The progress monitor. */
    private IProgressMonitor mMonitor;

    /** Map containing the file resources to audit. */
    private Map mFiles = new HashMap();

    /** Add the check rule name to the message? */
    private boolean mAddRuleName = false;

    /** Reference to the file buffer manager. */
    private final ITextFileBufferManager mFileBufferManager = FileBuffers
            .getTextFileBufferManager();

    // =================================================
    // Constructors & finalizer.
    // =================================================

    /**
     * Creates an auditor.
     * 
     * @param checkConfiguration the check configuraton to use during audit.
     */
    public Auditor(ICheckConfiguration checkConfiguration)
    {
        mCheckConfiguration = checkConfiguration;

        //
        // check wether to include rule names
        //
        Preferences prefs = CheckstylePlugin.getDefault().getPluginPreferences();
        mAddRuleName = prefs.getBoolean(CheckstylePlugin.PREF_INCLUDE_RULE_NAMES);
    }

    // =================================================
    // Methods.
    // =================================================

    /**
     * Runs the audit on the files associated with the auditor.
     * 
     * @param project the project is needed to build the correct classpath for
     *            the checker
     * @param monitor the progress monitor
     * @throws CheckstylePluginException error processing the audit
     */
    public void runAudit(IProject project, IProgressMonitor monitor)
        throws CheckstylePluginException
    {

        // skip if there are no files to check
        if (mFiles.isEmpty() || project == null)
        {
            return;
        }

        mMonitor = monitor;

        Checker checker = null;
        AuditListener listener = null;
        Filter runtimeExceptionFilter = null;

        // store the current context class loader
        ClassLoader contextClassloader = Thread.currentThread().getContextClassLoader();

        try
        {

            // get the classloader that is able to load classes from custom jars
            // within the extension-libraries dir
            ClassLoader customClassLoader = CustomLibrariesClassLoader.get();
            Thread.currentThread().setContextClassLoader(customClassLoader);

            File[] filesToAudit = getFileArray();

            // begin task
            monitor.beginTask(NLS.bind(Messages.Auditor_msgCheckingConfig, mCheckConfiguration
                    .getName()), filesToAudit.length);

            // set context
            mCheckConfiguration.setContext(project);

            // create checker
            checker = CheckerFactory.createChecker(mCheckConfiguration);

            // create and add listener
            listener = new CheckstyleAuditListener(project);
            checker.addListener(listener);

            // create and add filter for RuntimeExceptions reported by
            // RedundantThrowsCheck
            // BUGS 1177797, 996575
            runtimeExceptionFilter = new RuntimeExceptionFilter();
            checker.addFilter(runtimeExceptionFilter);

            // reconfigure the shared classloader for the current
            // project
            if (project.hasNature(JavaCore.NATURE_ID))
            {
                CheckerFactory.getSharedClassLoader().intializeWithProject(project);
            }

            // run the files through the checker
            checker.process(filesToAudit);

        }
        catch (IOException e)
        {
            CheckstylePluginException.rethrow(e);
        }
        catch (CoreException e)
        {
            CheckstylePluginException.rethrow(e);
        }
        catch (CheckstyleException e)
        {
            CheckstylePluginException.rethrow(e);
        }
        finally
        {
            monitor.done();

            // Cleanup listener and filter
            if (checker != null)
            {
                checker.removeListener(listener);
                checker.removeFilter(runtimeExceptionFilter);
            }

            // reset the context
            mCheckConfiguration.setContext(null);

            // restore the original classloader
            Thread.currentThread().setContextClassLoader(contextClassloader);
        }
    }

    /**
     * Add a file to the audit.
     * 
     * @param file the file
     */
    public void addFile(IFile file)
    {
        mFiles.put(file.getLocation().toOSString(), file);
    }

    /**
     * Get a file resource by the file name.
     * 
     * @param fileName the file name
     * @return the file resource or <code>null</code>
     */
    private IFile getFile(String fileName)
    {
        return (IFile) mFiles.get(fileName);
    }

    /**
     * Helper method to get an array of java.io.Files. This array gets passed to
     * the checker.
     * 
     * @return
     */
    private File[] getFileArray()
    {
        List files = new ArrayList();

        Collection iFiles = mFiles.values();
        Iterator it = iFiles.iterator();
        while (it.hasNext())
        {
            files.add(((IFile) it.next()).getLocation().toFile());
        }

        return (File[]) files.toArray(new File[files.size()]);
    }

    /**
     * Filter implementation to filter java.lang.RuntimeException errors that
     * are reportet from RedundantThrowsCheck.
     * 
     * @author Lars Kdderitzsch
     */
    private class RuntimeExceptionFilter implements Filter
    {

        /**
         * @see Filter#accept(com.puppycrawl.tools.checkstyle.api.AuditEvent)
         */
        public boolean accept(AuditEvent aEvent)
        {
            return aEvent.getMessage().indexOf(RuntimeException.class.getName()) == -1;
        }
    }

    /**
     * Implementation of the audit listener. This listener creates markers on
     * the file resources if checkstyle messages are reported.
     * 
     * @author David Schneider
     * @author Lars Kdderitzsch
     */
    private class CheckstyleAuditListener implements AuditListener
    {

        /** The tab width set within eclipse. */
        private int mTabWidth;

        /** the project. */
        private IProject mProject;

        /** The file currently being checked. */
        private IResource mResource;

        /** Document model of the current file. */
        private IDocument mDocument;

        /** internal counter used to time to actualisation of the monitor. */
        private int mMonitorCounter;

        /** map containing the marker data. */
        private Map mMarkerAttributes = new HashMap();

        /** flags if the amount of markers should be limited. */
        private boolean mLimitMarkers;

        /** the max amount of markers per resource. */
        private int mMarkerLimit;

        /** the count of markers generated for the current resource. */
        private int mMarkerCount;

        public CheckstyleAuditListener(IProject project)
        {
            mProject = project;

            try
            {

                // In Eclipse 3.1 there can be project specific settings for
                // tabwidtdh etc.
                // Here reflection is used to maintain compatibility with
                // Eclipse 3.0
                Method getTabWidht3_1Style = CodeFormatterUtil.class.getMethod("getTabWidth", //$NON-NLS-1$
                        new Class[] { IJavaProject.class });

                IJavaProject javaProject = JavaCore.create(project);

                Integer width = (Integer) getTabWidht3_1Style.invoke(null,
                        new Object[] { javaProject });
                mTabWidth = width.intValue();
            }
            catch (Exception e)
            {
                CheckstyleLog.log(e);
            }

            // init the marker limitation
            IPreferenceStore prefStore = CheckstylePlugin.getDefault().getPreferenceStore();
            mLimitMarkers = prefStore.getBoolean(CheckstylePlugin.PREF_LIMIT_MARKERS_PER_RESOURCE);
            mMarkerLimit = prefStore.getInt(CheckstylePlugin.PREF_MARKER_AMOUNT_LIMIT);
        }

        public void fileStarted(AuditEvent event)
        {
            // get the current IFile reference
            mResource = getFile(event.getFileName());
            mMarkerCount = 0;

            if (mResource != null)
            {

                // begin subtask
                if (mMonitorCounter == 0)
                {
                    mMonitor.subTask(NLS
                            .bind(Messages.Auditor_msgCheckingFile, mResource.getName()));
                }

                // increment monitor-counter
                this.mMonitorCounter++;

                // invalidate the last document
                mDocument = null;

            }
            else
            {

                IPath filePath = new Path(event.getFileName());
                IPath dirPath = filePath.removeFileExtension().removeLastSegments(1);

                IPath projectPath = mProject.getLocation();
                if (projectPath.isPrefixOf(dirPath))
                {
                    // find the resource with a project relative path
                    mResource = mProject.findMember(dirPath.removeFirstSegments(projectPath
                            .segmentCount()));
                }
                else
                {
                    // if the resource is not inside the project, take project
                    // as resource - this should not happen
                    mResource = mProject;
                }
            }
        }

        public void addError(AuditEvent error)
        {
            try
            {
                if (!mLimitMarkers || mMarkerCount < mMarkerLimit)
                {

                    SeverityLevel severity = error.getSeverityLevel();

                    if (!severity.equals(SeverityLevel.IGNORE) && mResource != null)
                    {

                        RuleMetadata metaData = MetadataFactory.getRuleMetadata(error
                                .getSourceName());
                        if (metaData != null)
                        {
                            mMarkerAttributes.put(CheckstyleMarker.MODULE_NAME, metaData
                                    .getInternalName());
                            mMarkerAttributes.put(CheckstyleMarker.MESSAGE_KEY, error
                                    .getLocalizedMessage().getKey());
                        }
                        mMarkerAttributes.put(IMarker.PRIORITY,
                                new Integer(IMarker.PRIORITY_NORMAL));
                        mMarkerAttributes.put(IMarker.SEVERITY, new Integer(
                                getSeverityValue(severity)));
                        MarkerUtilities.setLineNumber(mMarkerAttributes, error.getLine());
                        MarkerUtilities.setMessage(mMarkerAttributes, getMessage(error));

                        // calculate offset for editor annotations
                        calculateMarkerOffset(error, mMarkerAttributes);

                        // create a marker for the actual resource
                        MarkerUtilities.createMarker(mResource, mMarkerAttributes,
                                CheckstyleMarker.MARKER_ID);
                        mMarkerCount++;

                        // clear the marker attributes to reuse the map for the
                        // next error
                        mMarkerAttributes.clear();
                    }
                }
            }
            catch (CoreException e)
            {
                CheckstyleLog.log(e);
            }
        }

        public void addException(AuditEvent event, Throwable throwable)
        {
            CheckstyleLog.log(throwable);
        }

        public void fileFinished(AuditEvent event)
        {
            // update monitor according to the monitor interval
            if (mMonitorCounter == MONITOR_INTERVAL)
            {
                mMonitor.worked(MONITOR_INTERVAL);
                mMonitorCounter = 0;
            }
        }

        public void auditFinished(AuditEvent event)
        {}

        public void auditStarted(AuditEvent event)
        {}

        /**
         * Calculates the offset information for the editor annotations.
         * 
         * @param error the audit error
         * @param markerAttributes the marker attributes
         * @throws CoreException
         */
        private void calculateMarkerOffset(AuditEvent error, Map markerAttributes)
            throws CoreException
        {

            // lazy create the document for the current file
            if (mDocument == null && mResource instanceof IFile)
            {
                IPath path = mResource.getFullPath();
                mFileBufferManager.connect(path, new NullProgressMonitor());
                mDocument = mFileBufferManager.getTextFileBuffer(path).getDocument();
            }

            // Provide offset information for the marker to make
            // annotated source code possible
            if (mDocument != null)
            {
                try
                {

                    int line = error.getLine();

                    IRegion lineInformation = mDocument
                            .getLineInformation(line == 0 ? 0 : line - 1);
                    int lineOffset = lineInformation.getOffset();
                    int lineLength = lineInformation.getLength();

                    String lineData = mDocument.get(lineOffset, lineLength);

                    // annotate from the error column until the end of
                    // the line
                    int offset = getOffsetFromColumn(lineData, error.getColumn());

                    MarkerUtilities.setCharStart(markerAttributes, lineOffset + offset);
                    MarkerUtilities.setCharEnd(markerAttributes, lineOffset + lineLength);
                }
                catch (BadLocationException e)
                {
                    // seems to happen quite often so its no use to log since we
                    // can't do anything about it
                    // CheckstyleLog.log(e);
                }
            }
        }

        /**
         * Calculates the offset for the given column within this line. This is
         * done to get a correct offset if tab characters are used within this
         * line.
         * 
         * @param line the line as string
         * @param column the column
         * @return the true offset of this column within the line
         */
        private int getOffsetFromColumn(String line, int column)
        {

            int calculatedColumn = 0;

            int lineLength = line.length();
            for (int i = 0; i < lineLength; i++)
            {
                char c = line.charAt(i);
                if (c == '\t')
                {
                    calculatedColumn += mTabWidth;
                }
                else
                {
                    calculatedColumn++;
                }

                if (calculatedColumn >= column)
                {
                    return i;
                }
            }
            return lineLength;
        }

        private int getSeverityValue(SeverityLevel severity)
        {
            int result = IMarker.SEVERITY_WARNING;

            if (severity.equals(SeverityLevel.INFO))
            {
                result = IMarker.SEVERITY_INFO;
            }
            else if (severity.equals(SeverityLevel.WARNING))
            {
                result = IMarker.SEVERITY_WARNING;
            }
            else if (severity.equals(SeverityLevel.ERROR))
            {
                result = IMarker.SEVERITY_ERROR;
            }

            return result;
        }

        private String getMessage(AuditEvent error)
        {
            String message = error.getMessage();
            if (mAddRuleName)
            {
                StringBuffer buffer = new StringBuffer(getRuleName(error));
                buffer.append(": ").append(message); //$NON-NLS-1$
                message = buffer.toString();
            }
            return message;
        }

        private String getRuleName(AuditEvent error)
        {
            RuleMetadata metaData = MetadataFactory.getRuleMetadata(error.getSourceName());
            if (metaData == null)
            {
                return Messages.Auditor_txtUnknownModule;
            }
            return metaData.getRuleName();
        }
    }
}